/*******************************************************/
/*******************Map Handling Extended***************/
/*************************By AJX************************/
/*******************************************************/
/*
Notes:
This is the modified version of the original that uses 2 bytes to store each object, which as a consequence
takes double the space of the original when saving. However, this allowed for more than 255 different types
to be saved in one datum. The two bytes setup makes the type limit 65025.

It functions in the exact same way as the original, just like I said, bigger/slower/higher limit.


I STRONGLY advise against using this saving method if in any way avoidable. You'd be far better off just dividing
up the map you were saving into 4 parts to make yourself not cross the 255 limit threshold because saving this way
is simply much more inefficient.

The loading here takes nearly 15 times as long because of how it works. Saving isn't really affected, but as I
said loading definately takes a toll. I'm only releasing this because I said I would and I wanted to see how it
would function.
*/


/************************************/
/*  Don't touch from this point on  */
/*Unless you know what you are doing*/
/************************************/


/*******/


Map_Save/Extended

	IndexFind(atom/A)
		return Key.Find(A.type)
/*


*/

	IndexType(Num)
		return Key[Num]

	Index(atom/A)
		var/V=IndexFind(A)
		if(!V)
			if(Key.len==65025)
				world<<"\red AJX Debug: 65025 Key Limit, Error"
				return 0
			Key+=A.type
			V=Key.len
		var/VA=1+round(V/256)
		while(V>255) V-=255
		var/VB=V
		return "[ascii2text(VA)][ascii2text(VB)]"


	Load(turf/Loc)
		var/C
		var/C2
		var/Text
		var/Type
		var/X

		if(!Loc)Loc=locate(1,1,1)

		for(var/i=0,i<Map.len,i++)
			Text=Map[i+1]
			X=-1
			while(Text)
				C=copytext(Text,1,2)
				C2=copytext(Text,2,3)
				Text=copytext(Text,3)
				C=text2ascii(C)
				C2=text2ascii(C2)
				C=(C-1)*255+C2

				Type=IndexType(C)

				if(findtext("[Type]","/turf",1,6)) X++
				new Type(locate(Loc.x+X,Loc.y+i,1))



	Save(turf/Loc,Width,Height)	//Loc must be lower left corner
		if(!Loc)
			Loc=locate(1,1,1)
			Width=world.maxx
			Height=world.maxy
		if(!Width) return
		if(!Height) Height=Width
		var/turf/T
		var/X
		var/Y
		var/i
		var/atom/A
		var/Text=""
		Map.len=0
		Map.len=Height
		for(Y=0,Y<Height,Y++)

			for(X=0,X<Width,X++)
				T=locate(Loc.x+X,Loc.y+Y,Loc.z)

				Text="[Text][Index(T)]"
				for(i=1,i<=T.contents.len,i++)
					A=T.contents[i]
					if(A.A_DoNotSave) continue
					Text="[Text][Index(A)]"

			Map[Y+1]=Text
			Text=""